@@ -20,6 +20,7 @@ package POE::Component::Client::MPD::Item;
 use strict;
 use warnings;
 use POE::Component::Client::MPD::Item::Directory;
+use POE::Component::Client::MPD::Item::Playlist;
 use POE::Component::Client::MPD::Item::Song;
 
 #our ($VERSION) = '$Rev: 5645 $' =~ /(\d+)/;
@@ -34,10 +35,9 @@ sub new {
     my %lowcase;
     @lowcase{ map { lc } keys %params } = values %params;
 
-    my $item = exists $params{file} ?
-        POE::Component::Client::MPD::Item::Song->new(\%lowcase) :
-        POE::Component::Client::MPD::Item::Directory->new(\%lowcase);
-    return $item;
+    return POE::Component::Client::MPD::Item::Song->new(\%lowcase)      if exists $params{file};
+    return POE::Component::Client::MPD::Item::Directory->new(\%lowcase) if exists $params{directory};
+    return POE::Component::Client::MPD::Item::Playlist->new(\%lowcase) if exists $params{playlist};
 }
 
 1;
